class Embed:
    """
Nertivia HTMLEmbed

Attributes:
- title (str): The title of the embed.
- description (str): The description of the embed.
- colour (str): The colour of the embed.
- color (str): The color of the embed.
    """

    def __init__(self, title, description="", colour="#ffffff", color="#ffffff"):
        if colour == "":
            colour = color
        self.images = 0
        self.videos = 0
        self.footers = 0
        self.json = {
            "tag": "div",
            "styles": {
                "backgroundColor": f"rgb(0 0 0 / 15%)",
                "borderRadius": "4px",
                "display": "flex"
            },
            "content": [
                {
                    "tag": "div",
                    "styles": {
                        "backgroundColor": colour,
                        "flex": "left",
                        "width": "4px",
                        "borderRadius": "4px",
                        "flexShrink": 0
                    }
                },
                {
                    "tag": "div",
                    "styles": {
                        "backgroundColor": "inherit",
                        "flex": "right",
                        "width": "auto",
                        "padding": "10px",
                        "borderRadius": "4px"
                    },
                    "content": [
                        {
                            "tag": "strong", 
                            "styles": {
                                "fontSize": "16px"
                            },
                            "content": title
                        },
                        {
                            "tag": "div", 
                            "styles": {
                                "fontSize": "14px"
                            },
                            "content": description
                        }
                    ]
                }
            ]
        }

    def set_image(self, url):
        """
Set the image of the embed.

Args:
- url (str): The url of the image.
        """

        self.images += 1

        if self.images <= 1:
            self.json["content"][1]["content"][1]["content"] += "\n** **"
            
            self.json["content"][1]["content"].append({
                "tag": "img",
                "styles": {
                    "width": "100%",
                    "overflow": "hidden",
                    "borderRadius": "4px"
                },
                "attributes": {
                    "src": url
                }
            })

    def set_footer(self, text, icon_url=""):
        """
Set the footer of the embed.
    
Args:
- text (str): The text of the footer.
- icon_url (str): The url of the icon.
        """

        self.footers += 1
        if self.footers <= 1:
            if icon_url != "":
                self.json["content"][1]["content"].append({
                    "tag": "div",
                    "styles": {
                        "display": "flex",
                        "flexDirection": "row"
                    },
                    "content": [
                        {
                            "tag": "img",
                            "styles": {
                                "width": "20px",
                                "height": "20px",
                                "borderRadius": "4px",
                                "marginRight": "5px",
                                "marginTop": "5px"
                            },
                            "attributes": {
                                "src": icon_url
                            }
                        },
                        {
                            "tag": "span",
                            "styles": {
                                "fontSize": "12px",
                                "paddingTop": "10px"
                            },
                            "content": text
                        }
                    ]
                })
            else:
                self.json["content"][1]["content"].append({
                    "tag": "div",
                    "styles": {
                        "display": "flex",
                        "flexDirection": "row"
                    },
                    "content": [
                        {
                            "tag": "span",
                            "styles": {
                                "fontSize": "12px",
                                "paddingTop": "10px"
                            },
                            "content": text
                        }
                    ]
                })


import inspect

class Command:
    def __init__(self, name: str, description: str, usage: str, aliases: list, callback: callable = None, registered_with_function: bool = True) -> None:
        self.name = name
        self.description = description
        self.aliases = aliases
        self.usage = usage
        self.callback = callback
        self.registered_with_function = registered_with_function

        self.cog = None
        if "self" in self.callback.__code__.co_varnames:
            self.cog = self.callback.__code__.co_varnames[0]

    def get_callback(self) -> callable:
        return self.callback


from ..commands.command import Command

class Cog:
    def __init__(self, bot):
        self.bot = bot
        self.commands = []

    def add_command(self, **kwargs):
        description = kwargs["description"] if "description" in kwargs else ""
        usage = kwargs["usage"] if "usage" in kwargs else ""
        aliases = kwargs["aliases"] if "aliases" in kwargs else []
        name = kwargs["name"] if "name" in kwargs else "None"
        callback = kwargs["callback"] if "callback" in kwargs else None
        registered_with_function = kwargs["registered_with_function"] if "registered_with_function" in kwargs else True

        self.bot.register_command(**kwargs)
        self.commands.append(Command(name, description, usage, aliases, callback, registered_with_function=registered_with_function))

    def command(self, **kwargs):
        def decorator(func):
            self.add_command(**kwargs, callback=func)
            return func
        return decorator


from .user import *
from .bot import *
from .extra import *
from .textchannel import *
from .dmchannel import *
from .message import *
from .server import *
from .embed import *
from .button import *
from .member import *
from .get import *
from .colour import *

Color = Colour


import requests

from . import message
from . import extra
from . import embed
from . import user
from . import server

class TextChannel:
    """
Nertivia Text Channel

Attributes:
- id (int): The ID of the channel.
- name (str): The name of the channel.
- server_id (int): The ID of the server.
    """

    def __init__(self, id, name="", server_id="") -> None:
        if name == "" or server_id == "":
            response = requests.get(f"https://nertivia.net/api/channels/{id}", headers={"authorization": extra.Extra.getauthtoken()})

            self.id = response.json()["channelId"]
            self.name = response.json()["name"]
            self.server = server.Server(response.json()["server_id"])
        
        else:
            self.id = id
            self.name = name
            self.server = server.Server(server_id)

    def __str__(self) -> str:
        return self.name

    def send(self, content = "", embed: embed.Embed = None, buttons: list = None) -> message.Message:
        """
Sends a message to the channel.

Args:
- content (str): The content of the message.
- embed (embed.Embed): The embed of the message.
- buttons (list): A list of buttons to add to the message.

Aliases:
- send_message(content, embed, buttons)

Returns:
- message.Message: The message that was sent.
        """

        content = str(content)
        body={}

        if content != "":
            body["message"] = content

        if embed != None:
            body["htmlEmbed"] = embed.json

        if buttons != None:
            body["buttons"] = []
            for button in buttons:
                body["buttons"].append(button.json)

        response = requests.post(
            f"https://nertivia.net/api/messages/channels/{self.id}",
            headers={"authorization": extra.Extra.getauthtoken()},
            json=body
        )

        if "messagecreated" not in response.text.lower():
            return False

        return message.Message(response.json()["messageCreated"]["messageID"], self.id)

    def edit(self, name) -> dict:
        """
Edits the channel

Args:
- name (str): The new name of the channel.

Returns:
- dict: The response of the request.
        """

        response = requests.patch(
            f"https://nertivia.net/api/servers/{self.server_id}/channels/{self.id}",
            headers={"authorization": extra.Extra.getauthtoken()},
            json={
                "name": name
            }
        )

        self.name = name

        return response.json()

    def delete(self) -> dict:
        """
Deletes the channel.

Returns:
- dict: The response of the request.
        """

        response = requests.delete(
            f"https://nertivia.net/api/servers/{self.server_id}/channels/{self.id}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        return response.json()

    def typing(self):
        """
Tells the channel that the user is typing.

Returns:
- requests response: The response of the request.
        """

        response = requests.post(
            f"https://nertivia.net/api/messages/{self.id}/typing",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        return response

    def get_messages(self, amount: int = 1) -> list:
        """
Gets the messages from the channel.

Args:
- amount (int): The amount of messages to get.

Returns:
- list: The messages.
        """

        messages = []
        index = 0
        response = requests.get(
            f"https://nertivia.net/api/messages/channels/{self.id}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        for item in response.json()["messages"]:
            index += 1
            try:
                author = user.User(item["creator"]["id"], item["creator"]["username"], item["creator"]["tag"], item["creator"]["avatar"])
                message = message.Message(item["messageID"], self.id, author, item["message"], item["created"])
                messages.append(message)
            except:
                pass

            if index == amount:
                break
        
        return messages

    def get_message(self, id):
        """
Gets a message from the channel.

Args:
- id (int): The ID of the message.

Returns:
- message.Message: The message.
- None: If the message doesn't exist.
        """

        messages = self.getMessages()
        for message in messages:
            if message.id == id:
                return message
        
        return None

    send_message = send


class Events:
    def __init__(self):
        self.events = []
        self.custom_events = []

        self.custom_events.append("on_command")
        self.custom_events.append("on_command_error")

        self.events.append({"on_success": "authenticated"})
        self.events.append({"on_ready": "authenticated"})
        self.events.append({"on_auth_error": "authentication_error"})

        self.events.append({"on_message": "message:created"})
        self.events.append({"on_message_delete": "message:deleted"})
        self.events.append({"on_message_bulk_delete": "message:deleted_bulk"})
        self.events.append({"on_message_edit": "message:updated"})

        # self.events.append({"on_reaction_add": "message:add_reaction"})
        self.events.append({"on_reaction_update": "message:reaction_updated"})

        self.events.append({"on_notification_dismiss": "notification:dismiss"})

        self.events.append({"on_channel_create": "channel:created"})
        self.events.append({"on_channel_remove": "channel:removed"})
        self.events.append({"on_channel_mute": "channel:muted"}) 
        self.events.append({"on_channel_unmute": "channel:unmuted"})

        self.events.append({"on_server_channel_create": "server:channel_created"})
        self.events.append({"on_server_channel_remove": "server:channel_deleted"})
        self.events.append({"on_server_channel_update": "server:channel_updated"})
        self.events.append({"on_server_channel_position_update": "server:channel_position_updated"})
        
        self.events.append({"on_server_position_update": "self:position_updated"})
        self.events.append({"on_server_join": "server:joined"})
        self.events.append({"on_server_leave": "server:left"})
        self.events.append({"on_server_mute": "server:muted"})

        self.events.append({"on_server_members": "server:members"})
        self.events.append({"on_server_roles": "server:roles"})

        self.events.append({"on_roles_update": "server:roles_updated"})
        self.events.append({"on_role_create": "server:role_created"})
        self.events.append({"on_role_delete": "server:role_deleted"})
        self.events.append({"on_role_update": "server:role_updated"})

        self.events.append({"on_server_member_role_remove": "server:role_removed_from_member"})
        self.events.append({"on_server_member_role_add": "server:role_added_to_member"})

        self.events.append({"on_member_join": "server:member_added"})
        self.events.append({"on_member_leave": "server:member_removed"})

        self.events.append({"on_self_status_update": "user:status_changed"})
        self.events.append({"on_self_custom_status_update": "user:custom_status_changed"})
        self.events.append({"on_self_program_activity_update": "user:program_activity_changed"})

        self.events.append({"on_user_blocked": "user:blocked"})
        self.events.append({"on_user_unblocked": "user:unblocked"})

        self.events.append({"on_user_joined_call": "user:call_joined"})
        self.events.append({"on_user_left_call": "user:call_left"})
        self.events.append({"on_voice_receive_signal": "voice:signal_received"})
        self.events.append({"on_voice_receive_return_signla": "voice:return_signal_received"})

        self.events.append({"on_google_drive_linked": "google_drive:linked"})

        self.events.append({"on_user_status_update": "user_status_change"})
        self.events.append({"on_user_custom_status_update": "member:custom_status_change"})

        self.events.append({"on_relationship_add": "relationship:added"})
        self.events.append({"on_relationship_remove": "relationship:removed"})
        self.events.append({'on_relationship_accept': 'relationship:accepted'})

        self.events.append({"on_custom_emoji_renamed": "custom_emoji:renamed"})
        self.events.append({"on_custom_emoji_uploaded": "custom_emoji:uploaded"})
        self.events.append({"on_custom_emoji_deleted": "custom_emoji:removed"})

        self.events.append({"on_button_click": "message_button_clicked"})
        self.events.append({"on_button_click_callback": "message_button_click_callback"})

    def get_event(self, event_name):
        for event in self.events:
            for key, value in event.items():
                if key == event_name:
                    return value

    def get_event_name(self, event_value):
        for event in self.events:
            for key, value in event.items():
                if value == event_value:
                    return key

    def is_valid_event(self, event_name):
        for event in self.events:
            for key, value in event.items():
                if key == event_name:
                    return True
        return False


import requests

from . import extra
from . import user

class Bot:
    """
Nertivia Bot User

Attributes:
- id (int): The ID of the bot.
- username (str): The username of the bot.
- tag (str): The tag of the bot.
- avatar (str): The avatar of the bot.
- commands (list): The commands of the bot.
- creator (user.User): The creator of the bot.
    """

    def __init__(self, id, username="", tag="", avatar="", commands=[], creator = None) -> None:
        if username == "" or tag == "" or avatar == "" or commands == [] or creator is None:
            response = requests.get(
                f"https://nertivia.net/api/bots/{id}",
                headers={"Authorization": extra.Extra.getauthtoken()}
            )
            
            self.id = response.json()["id"]
            self.username = response.json()["username"]
            self.tag = response.json()["tag"]
            self.avatar = response.json()["avatar"]
            self.commands = response.json()["botCommands"]
            self.creator = user.User(response.json()["createdBy"]["id"])

        else:
            self.id = id
            self.username = username
            self.tag = tag
            self.avatar = avatar
            self.commands = commands
            self.creator = creator

    def __str__(self):
        return f"{self.username}:{self.tag}"

    def delete(self) -> dict:
        """
Deletes the bot.

Returns:
    dict: The response of the request.
        """

        response = requests.delete(
            "https://nertivia.net/api/bots/" + str(self.id),
            headers={"Authorization": self.token}
        )

        return response.json()


from . import server
from . import user

class Member:
    """
Nertivia Member
The same as user.User but has ban and kick functions.

Attributes:
- id (int): The ID of the member.
- username (str): The username of the member.
- tag (str): The tag of the member.
- avatar (str): The avatar of the member.
- type (str): The type of the member.
- server_id (int): The ID of the server the member is in.
    """

    def __init__(self, id, username="", tag="", avatar="", type="", server_id="") -> None:
        self.id = id
        self.username = username
        self.tag = tag
        self.avatar = avatar
        self.type = type
        self.server_id = server_id

    def __str__(self) -> str:
        return f"{self.username}:{self.tag}"

    def ban(self):
        """
Bans the member.

Returns:
- bool: Whether the member was banned or not.
        """

        server2 = server.Server(self.server_id)
        return server2.ban_member(user.User(self.id, self.username, self.tag, self.avatar))
    
    def kick(self):
        """
Kicks the member.

Returns:
- bool: Whether the member was kicked or not.
        """

        server2 = server.Server(self.server_id)
        return server2.ban_member(user.User(self.id, self.username, self.tag, self.avatar))


"""
# Nertivia4PY
A Python wrapper for the Nertivia API.  

Checkout the Github for examples and documentation.  
https://github.com/bentettmar/nertivia4py
"""

from .bot import *

from .utils import get
from .commands import Cog


import socketio
import shlex
import requests
import importlib

from .gateway import events

from .commands import command

from .utils import user
from .utils import extra
from .utils import message
from .utils import exceptions

class Bot:
    """
Nertivia Gateway Client  
This is the client that handles all of the events from the Nertivia Gateway.  
And it also handles all of the commands that are sent to the bot.  

Attributes:
- command_prefix (str): The prefix that is used to identify commands.
- self_bot (bool): Whether or not the bot is a self bot.
- debug (bool): Whether or not to send socket debug messages.
    """

    def __init__(self, command_prefix, self_bot=False, debug=False) -> None:
        self.socket = socketio.Client(engineio_logger=False, logger=debug)
        self.socket_ip = "https://nertivia.net/"

        self.command_prefix = command_prefix
        self.commands = []

        self.user = None
        self.token = ""
        self.self_bot = self_bot

        self.other_settings = {
            "on_command_callback": None,
            "on_command_error_callback": None,
            "on_message_callback": None
        }

        extra.Extra.setauthtoken(self.token)

    def run(self, token) -> None:
        """
Run the client  

Args:
- token (str): The token that is used to authenticate the bot.

Raises:
- InvalidToken: If the token is invalid.
        """

        extra.Extra.setauthtoken(token)

        self.token = token

        user_response = requests.get("https://nertivia.net/api/user", headers={"authorization": extra.Extra.getauthtoken()})
        if user_response.status_code != 200:
            raise exceptions.InvalidToken("Token is invalid.")

        user_obj = user_response.json()["user"]

        self.user = user.User(user_obj["id"])

        self.socket.connect(self.socket_ip, namespaces=["/"], transports=["websocket"])
        self.socket.emit("authentication", {"token": token})
        self.socket.wait()

    def _on_message_event_handler(self, event):
        msg = message.Message(event["message"]["messageID"], event["message"]["channelId"])
        command_can_be_run = False
        is_a_command = False
        
        if self.other_settings["on_message_callback"] is not None:
            self.other_settings["on_message_callback"](msg)

        if self.self_bot:
            if msg.creator.id == self.user.id:
                command_can_be_run = True
        else:
            if msg.creator.id != self.user.id:
                command_can_be_run = True

        if msg.content.startswith(self.command_prefix):
            is_a_command = True

        if is_a_command:
            command = msg.content.replace(self.command_prefix, "")
            args = shlex.split(command)
            command = args[0]
            args.pop(0)

        if command_can_be_run:
            if is_a_command:
                for cmd in self.commands:
                    if command == cmd.name or command in cmd.aliases:
                        callback = cmd.get_callback()

                        try: 
                            if cmd.cog is not None and cmd.registered_with_function is False: callback(cmd.cog, msg, args)
                            else: callback(msg, args)

                        except TypeError: 
                            if cmd.cog is not None and cmd.registered_with_function is False: callback(cmd.cog, msg)
                            else: callback(msg)

                        except Exception as e: 
                            if self.other_settings["on_command_error_callback"] is not None:
                                self.other_settings["on_command_error_callback"](msg, str(e))

                            else:
                                raise exceptions.CommandError(str(e))

                        try:
                            if self.other_settings["on_command_callback"] is not None:
                                self.other_settings["on_command_callback"](msg, cmd)

                        except Exception as e:
                            print(e)

        if is_a_command and command_can_be_run:
            if command not in [cmd.name for cmd in self.commands]:
                if self.other_settings["on_command_error_callback"] is not None:
                    self.other_settings["on_command_error_callback"](msg, exceptions.CommandNotFound(f"Command {command} not found."))

    def register_command(self, **kwargs) -> None:
        """
Register a command.  

Args:
- name (str): The name of the command.
- description (str): The description of the command.
- usage (str): The usage of the command.
- aliases (list): A list of aliases for the command.
- callback (func): The callback function for the command. / What the command does.

Aliases:
- add_command(**kwargs)

Raises:
- CommandAlreadyExists: If the command already exists.
        """

        description = kwargs["description"] if "description" in kwargs else ""
        usage = kwargs["usage"] if "usage" in kwargs else ""
        aliases = kwargs["aliases"] if "aliases" in kwargs else []
        callback = kwargs["callback"] or kwargs["func"] or kwargs["function"]
        name = kwargs["name"] if "name" in kwargs else callback.__name__
        registered_with_function = kwargs["registered_with_function"] if "registered_with_function" in kwargs else True

        for cmd in self.commands:
            if name in aliases:
                raise ValueError("Command name and aliases cannot be the same.")

            if cmd.name == name:
                raise exceptions.CommandAlreadyExists(f"Command name {name} already exists.")

        self.commands.append(command.Command(name, description, usage, aliases, callback, registered_with_function))
        self.socket.on(events.Events().get_event("on_message"), self._on_message_event_handler)

    def load_commands(self, **kwargs):
        """
Load commands from a separate file.

Args:
- lib_path (str): The path to the file. (example: commands.general)
- commands_class (str): The name of the class in the file. (example: Commands)

Aliases:
- add_commands(lib_path, commands_class)

Raises:
- AttributeError: If the class is not found.
        """
    
        if "lib_path" in kwargs:  lib_path = kwargs["lib_path"]
        else: raise AttributeError("lib_path is required.")

        if "commands_class" in kwargs: commands_class = kwargs["commands_class"]
        else: raise AttributeError("commands_class is required.")

        lib = importlib.import_module(lib_path)
        commands_class = getattr(lib, commands_class)
        class_instance = commands_class(self)
        class_instance.register()

    def event(self, *args):
        event_name = args[0].__name__
        event_callback = args[0]
        custom_events = events.Events().custom_events

        if events.Events().is_valid_event(event_name) or event_name in custom_events:
            if event_name == "on_message":
                self.other_settings["on_message_callback"] = event_callback
                self.socket.on(events.Events().get_event(event_name), self._on_message_event_handler)

            elif event_name == "on_command":
                self.other_settings["on_command_callback"] = event_callback
                self.socket.on(events.Events().get_event("on_message"), self._on_message_event_handler)

            elif event_name == "on_command_error":
                self.other_settings["on_command_error_callback"] = event_callback
                self.socket.on(events.Events().get_event("on_message"), self._on_message_event_handler)

            else:
                event = events.Events().get_event(event_name)
                self.socket.on(event, event_callback)

        else:
            raise exceptions.InvalidEvent("Event name is invalid.")

    def command(self, **kwargs):
        def decorator(func):
            command_name = kwargs["name"] if "name" in kwargs else func.__name__
            command_description = kwargs["description"] if "description" in kwargs else ""
            command_usage = kwargs["usage"] if "usage" in kwargs else ""
            command_aliases = kwargs["aliases"] if "aliases" in kwargs else []

            self.register_command(name=command_name, description=command_description, usage=command_usage, aliases=command_aliases, callback=func)

            return func
        return decorator

    add_command = register_command
    add_commands = load_commands
    register_commands = load_commands


from . import server
from . import textchannel
from . import user

class Get:
    
    def get_server(id) -> server.Server:
        """
Gets a Nertivia server.

Args:
- id (int): The ID of the server.

Returns:
- server.Server: The server.
        """

        return server.Server(id)

    def get_text_channel(id) -> textchannel.TextChannel:
        """
Gets a Nertivia text channel.

Args:
- id (int): The ID of the text channel.

Returns:
- textchannel.TextChannel: The text channel.
        """

        return textchannel.TextChannel(id)

    def get_user(id) -> user.User:
        """
Gets a Nertivia user.

Args:
- id (int): The ID of the user.

Returns:
- user.User: The user.
        """

        return user.User(id)


import random

from enum import Enum

def random_colour():
    colour = lambda: random.randint(0,255)
    return '#%02X%02X%02X' % (colour(), colour(), colour())

class Colour(Enum):
    """
Default colours to use in Embeds or as strings.

Colours:
- Blue : BLUE
- Green : GREEN
- Yellow : YELLOW
- Pink : PINK
- Red : RED
- Black : BLACK
- White : WHITE
- Random : RANDOM
    """

    def __str__(self):
        return str(self.value)

    BLUE = "#5865F2"
    GREEN = "#57F287"
    YELLOW = "#FEE75C"
    PINK = "#EB459E"
    RED = "#ED4245"
    BLACK = "#000000"
    WHITE = "#FFFFFF"
    RANDOM = random_colour()



import requests

from . import embed
from . import extra

class DMChannel:
    """
Nertivia DM Channel
Same as Text Channel but is used to send messages to a user.

Attributes:
- id (int): The ID of the channel.
    """

    def __init__(self, id) -> None: 
        self.id = id

    def send(self, content = "", embed: embed.Embed = None, buttons: list = None) -> dict:
        """
Sends a message to the channel.

Args:
- content (str): The content of the message.
- embed (embed.Embed): The embed of the message.
- buttons (list): The buttons of the message.

Aliases:
- send_message(content, embed, buttons)

Returns:
- dict: The response of the request.
        """

        content = str(content)
        body = {}
        if content != "":
            body["message"] = content
        if embed is not None:
            body["htmlEmbed"] = embed.json
        if buttons != None:
            body["buttons"] = []
            for button in buttons:
                body["buttons"].append(button.json)

        response = requests.post(f"https://nertivia.net/api/messages/channels/{self.id}", headers={"authorization": extra.Extra.getauthtoken()}, json=body)
        return response.json()

    send_message = send


import requests
import datetime
import socketio
import ast
import json

from nertivia4py.gateway import events
from . import textchannel
from . import user
from . import extra

class Server:
    """
Nertivia Server

Attributes:
- id (str): The server's ID.
- name (str): The server's name.
- avatar (str): The server's avatar.
- default_channel_id (str): The server's default channel ID.
- created (str): The server's creation date.
- banner (str): The server's banner.
    """

    def __init__(self, id, name="", avatar="", default_channel_id="", created="", banner="") -> None:
        if name == "" or avatar == "" or default_channel_id == "" or created == "" or banner == "":
            response = requests.get(
                f"https://nertivia.net/api/servers/{id}",
                headers={"authorization": extra.Extra.getauthtoken()}
            )

            self.id = response.json()["server_id"]
            self.name = response.json()["name"]
            self.avatar = response.json()["avatar"]
            self.default_channel_id = response.json()["default_channel_id"]
            self.created = response.json()["created"]
            self.created_at = datetime.datetime.fromtimestamp(int(self.created) / 1000)

        else:
            self.id = id
            self.name = name
            self.avatar = avatar
            self.default_channel_id = default_channel_id
            self.created = created
            self.created_at = datetime.datetime.fromtimestamp(int(self.created) / 1000)
            # self.banner = banner

        self.members = []
        self.channels = []

    def __str__(self):
        return f"{self.name}"

    def edit(self, name) -> dict:
        """
Edits the server

Args:
- name (str): The new name of the server.

Returns:
- dict: The response of the request.
        """

        response = requests.patch(
            f"https://nertivia.net/api/servers/{self.id}",
            headers={"authorization": extra.Extra.getauthtoken()},
            json={
                "name": name
            }
        )

        self.name = response.json()["name"]
        return response.json()

    def delete(self) -> dict:
        """
Deletes the server

Returns:
- dict: The response of the request.
        """

        response = requests.delete(
            f"https://nertivia.net/api/servers/{self.id}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        return response.json()

    def get_bans(self):
        """
Gets a list of banned users from the server

Returns:
- list: A list of banned users.
- None: If the request failed.
        """

        response = requests.get(
            f"https://nertivia.net/api/servers/{self.id}/bans",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        if "missing permission" not in response.text.lower():
            bans = []

            for user3 in response.json():
                user4 = user3["user"]
                user2 = user.User(user["id"])
                bans.append(user2)

            return bans
        
        else:
            return False

    def ban_member(self, user: user.User) -> bool:
        """
Bans a member from the server

Args:
- user (user.User): The user to ban.

Returns:
- bool: Whether the request was successful.
        """

        userId = user.id
        response = requests.put(
            f"https://nertivia.net/api/servers/{self.id}/bans/{userId}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        if "missing permission" not in response.text.lower():
            return True
        else:
            return False

    def kick_member(self, user: user.User) -> bool:
        """
Kicks a member from the server

Args:
- user (user.User): The user to kick.

Returns:
- bool: Whether the request was successful.
        """

        userId = user.id
        response = requests.delete(
            f"https://nertivia.net/api/servers/{self.id}/members/{userId}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        if "missing permission" not in response.text.lower():
            return True
        else:
            return False

    def unban_member(self, user: user.User) -> dict:
        """
Unbans a member from the server

Args:
- user (user.User): The user to unban.

Returns:
- dict: The response of the request.
        """

        userId = user.id
        response = requests.delete(
            f"https://nertivia.net/api/servers/{self.id}/bans/{userId}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        return response.json()

    def _get_members_handler(self, event):
        data = ast.literal_eval(str(event))

        members = []
        for servermember in data["serverMembers"]:
            if servermember["server_id"] == str(self.id):
                members.append(servermember)

        self.members = members

    def get_members(self) -> list:
        """
Gets a list of members from the server

Aliases:
- get_all_members()

Returns:
- list: A list of members.
        """

        socket = socketio.Client()
        socket.connect("https://nertivia.net/", namespaces=["/"], transports=["websocket"])
        socket.emit("authentication", {"token": extra.Extra.getauthtoken()})

        socket.on(events.Events().get_event("on_success"), self._get_members_handler)

        while len(self.members) == 0:
            pass

        socket.disconnect()

        return self.members

    def _get_channels_handler(self, event):
        data = ast.literal_eval(str(event))
        servers = data["user"]["servers"]
        channels = []

        for server in servers:
            if server["server_id"] == self.id:
                for chnl in server["channels"]:
                    chnl2 = textchannel.TextChannel(chnl["channelID"], chnl["name"], chnl["server_id"])
                    channels.append(chnl2)

        self.channels = channels

    def get_channels(self) -> list:
        """
Gets a list of channels from the server

Aliases:
- get_all_channels()

Returns:
- list: A list of channels.
        """

        socket = socketio.Client()
        socket.connect("https://nertivia.net/", namespaces=["/"], transports=["websocket"])
        socket.emit("authentication", {"token": extra.Extra.getauthtoken()})

        socket.on(events.Events().get_event("on_success"), self._get_channels_handler)

        while len(self.channels) == 0:
            pass

        socket.disconnect()

        return self.channels

    def create_text_channel(self, name):
        """
Creates a text channel in the server.

Args:
- name (str): The name of the channel.

Returns:
- textchannel.TextChannel: The channel that was created.
        """

        response = requests.put(
            f"https://nertivia.net/api/servers/{self.id}/channels",
            headers={"authorization": extra.Extra.getauthtoken()},
            json={
                "name": name,
                "type": 1
            }
        )

        return textchannel.TextChannel(response.json()["channel"]["channelId"])

    get_all_members = get_members
    get_all_channels = get_channels


import requests

from . import embed
from . import user
from . import extra
from . import textchannel
from . import dmchannel

class Message:
    """
Nertivia Message

Attributes:
- id (int): The ID of the message.
- channel (textchannel.TextChannel): The channel the message was sent in.
- creator (user.User): The author of the message.
- content (str): The content of the message.
- created (str): The time the message was created.
    """

    def __init__(self, id, channelId, creator="", content="", created="") -> None:
        if creator == "" or content == "" or created == "":
            response = requests.get(
                f"https://nertivia.net/api/messages/{id}/channels/{channelId}",
                headers={"authorization": extra.Extra.getauthtoken()}
            )

            channel_response = requests.get(
                f"https://nertivia.net/api/channels/{response.json()['channelId']}",
                headers={"authorization": extra.Extra.getauthtoken()}
            )

            if "recipients" in channel_response.json():
                self.channel = dmchannel.DMChannel(response.json()["channelId"])
            else:
                self.channel = textchannel.TextChannel(response.json()["channelId"])

            self.id = response.json()["messageID"]
            # check if response json has a key called message and if it does then set self.content to that value
            try:
                self.content = response.json()["message"]
            except:
                self.content = ""
            try:
                self.created = response.json()["created"]
            except:
                self.created = ""
            try:
                self.creator = user.User(response.json()["creator"]["id"])
            except:
                self.creator = None

        else:
            self.id = id
            self.channel = textchannel.TextChannel(channelId)
            self.content = content
            self.created = created
            self.creator = creator

    def __str__(self) -> str:
        return self.content

    def reply(self, content:str = "", embed: embed.Embed = None, buttons: list = None) -> "Message":
        """
Replies to the message.

Args:
- content (str): The content of the message.
- embed (embed.Embed): The embed of the message.
- buttons (list of button.Button): A list of buttons to add to the message.

Returns:
- Message: The message that was sent.
        """

        body={"message": f"<m{self.id}>"+content}
        if embed != None:
            body["htmlEmbed"] = embed.json
        if buttons != None:
            body["buttons"] = []
            for button in buttons:
                body["buttons"].append(button.json)

        response = requests.post(
            f"https://nertivia.net/api/messages/channels/{self.channel.id}",
            headers={"authorization": extra.Extra.getauthtoken()},
            json=body
        )

        return Message(response.json()["messageCreated"]["messageID"], self.channel.id)

    def edit(self, content, embed: embed.Embed = None, buttons: list = None) -> dict:
        """
Edits the message.

Args:
- content (str): The content of the message.
- embed (embed.Embed): The embed of the message.
- buttons (list of button.Button): A list of buttons to add to the message.

Returns:
- dict: The response of the request.
        """

        content = str(content)
        body = {"message": content}
        if embed is not None:
            body["htmlEmbed"] = embed.json
        if buttons is not None:
            body["buttons"] = []
            for button in buttons:
                body["buttons"].append(button.json)

        response = requests.patch(
            f"https://nertivia.net/api/messages/{self.id}/channels/{self.channel.id}",
            headers={"authorization": extra.Extra.getauthtoken()},
            json=body
        )

        return response.json()

    def delete(self) -> dict:
        """
Deletes the message.
    
Returns:
- dict: The response of the request.
        """

        response = requests.delete(
            f"https://nertivia.net/api/messages/{self.id}/channels/{self.channel.id}",
            headers={"authorization": extra.Extra.getauthtoken()}
        )

        return response.json()

    def add_reaction(self, emoji) -> dict:
        """
Adds a reaction to the message.

Args:
- emoji (str): The emoji to add.

Returns:
- dict: The response of the request.
        """

        response = requests.post(
            f"https://nertivia.net/api/messages/{self.id}/channels/{self.channel.id}/reactions",
            headers={"authorization": extra.Extra.getauthtoken()},
            json={"unicode": emoji, "gif": False}
        )

        return response.json()

    def remove_reaction(self, emoji) -> dict:
        """
Removes a reaction to the message.

Args:
- emoji (str): The emoji to remove.

Returns:
- dict: The response of the request.
        """

        response = requests.delete(
            f"https://nertivia.net/api/messages/{self.id}/channels/{self.channel.id}/reactions",
            headers={"authorization": extra.Extra.getauthtoken()},
            json={"unicode": emoji}
        )

        return response.json()


class Button:
    """
Nertivia Button

Attributes:
- name (str): The name of the button.
- id (int): The id of the button.
    """

    def __init__(self, name, id) -> None:
        self.name = name
        self.id = id
        self.json = {
            "name": name,
            "id": id
        }

    def __str__(self) -> str:
        return f"{self.name}:{self.id}"


from distutils.core import setup

readme = """
# Nertivia4PY
A Python wrapper for the Nertivia API.  
Support Nertivia server : https://nertivia.net/i/nertivia4py

> ### Install
> ```
> pip install nertivia4py
> ```

> ### Example 
> ```python
> import nertivia4py
> 
> token = "TOKEN_HERE"
> prefix = "!"
> 
> bot = nertivia4py.Bot(prefix)
> 
> @bot.event
> def on_success(event):
>     print("Connected!")
> 
> @bot.command(name="ping", description="Ping command.")
> def ping_command(message, args):
>     message.reply("Pong!")
> 
> bot.run(token)
> ```  
> 
> For more examples, take a look at the examples folder in the github repo.
"""

setup(
    name='nertivia4py',
    packages=['nertivia4py', 'nertivia4py.gateway', 'nertivia4py.utils', 'nertivia4py.commands'],
    version='1.0.8',
    license='MIT',
    description='A Python wrapper for the Nertivia API',
    long_description_content_type="text/markdown",
    long_description=readme,
    author='Ben Tettmar',
    author_email='hello@benny.fun',
    url='https://github.com/bentettmar/nertivia4py',
    keywords=["nertivia", "api", "wrapper", "python",
              "bot", "nertivia.py", "nertivia4py"],
    install_requires=["requests", 'python-socketio[client]'],
)



import requests

from . import embed
from . import extra
from . import dmchannel

class User:
    """
Nertivia User

Attributes:
- id (int): The ID of the user.
- username (str): The username of the user.
- tag (str): The tag of the user.
- avatar (str): The avatar of the user.
- banner (str): The banner of the user.
- created (str): The date the user was created.
- blocked (bool): Whether the user is blocked or not.
    """

    def __init__(self, id, username="", tag="", avatar="", banner="", created="", blocked="") -> None:
        if username == "" or tag == "" or avatar == "":
            response = requests.get(f"https://nertivia.net/api/user/{id}", headers={"Authorization": extra.Extra.getauthtoken()})
            self.id = response.json()["user"]["id"]
            self.avatar = response.json()["user"]["avatar"]
            try:
                self.banner = response.json()["user"]["banner"]
            except:
                pass
            self.username = response.json()["user"]["username"]
            self.tag = response.json()["user"]["tag"]
            try:
                self.created = response.json()["user"]["created"]
            except:
                pass
            try:
                self.blocked = response.json()["isBlocked"]
            except:
                pass
        else:
            self.id = id
            self.avatar = avatar
            self.banner = banner
            self.username = username
            self.tag = tag
            self.created = created
            self.blocked = blocked
        self.avatar_url = f"https://media.nertivia.net/{self.avatar}"
        self.mention = f"[@:{self.id}]"

    def __str__(self) -> str:
        return f"{self.username}:{self.tag}"
    
    def __repr__(self) -> str:
        return f"{self.username}:{self.tag}"
    
    def send_friend_request(self) -> dict:
        """
        Send a friend request to the user.

        Returns:
        - dict: The response of the request.
        """

        response = requests.post(
            "https://nertivia.net/api/user/relationship",
            headers={
                "Authorization": extra.Extra.getauthtoken(),
                "Content-Type": "application/json"
            },
            json={
                "username": self.username,
                "tag": self.tag
            }
        )

        return response.json()

    def accept_friend_request(self) -> dict:
        """
Accept a friend request from the user.

Returns:
- dict: The response of the request.
        """

        response = requests.put(
            "https://nertivia.net/api/user/relationship",
            headers={
                "Authorization": extra.Extra.getauthtoken(),
                "Content-Type": "application/json"
            },
            json={
                "id": self.id
            }
        )
        
        return response.json()

    def decline_friend_request(self) -> dict:
        """
Decline a friend request from the user.

Returns:
- dict: The response of the request.
        """

        response = requests.delete(
            "https://nertivia.net/api/user/relationship",
            headers={
                "Authorization": extra.Extra.getauthtoken(),
                "Content-Type": "application/json"
            },
            json={
                "id": self.id
            }
        )

        return response.json()

    def block(self) -> dict:
        """
        Block the user.

        Returns:
        - dict: The response of the request.
        """

        response = requests.post(
            "https://nertivia.net/api/user/block",
            headers={
                "Authorization": extra.Extra.getauthtoken(),
                "Content-Type": "application/json"
            },
            json={
                "id": self.id
            }
        )

        return response.json()

    def unblock(self) -> dict:
        """
Unblock the user.

Returns:
- dict: The response of the request.
        """

        response = requests.delete(
            "https://nertivia.net/api/user/block",
            headers={
                "Authorization": extra.Extra.getauthtoken(),
                "Content-Type": "application/json"
            },
            json={
                "id": self.id
            }
        )

        return response.json()

    def dm(self, message, embed: embed.Embed = None) -> dict:
        """
Send a direct message to the user.

Args:
- message (str): The message to send.
- embed (embed.Embed): The embed to send.

Aliases:
- send_message(message, embed)
- send_dm(message, embed)
- send(message, embed)

Returns:
- dict: The response of the request.
        """

        response = requests.post(
            f"https://nertivia.net/api/channels/{self.id}",
            headers={
                "Authorization": extra.Extra.getauthtoken(),
                "Content-Type": "application/json"
            }
        )

        channel = dmchannel.DMChannel(response.json()["channel"]["channelId"])
        
        return channel.send(message, embed=embed)
    
    send_message = dm
    send_dm = dm
    send = dm